#!/bin/sh

#
# generic bfagent install script
#
# copyright @ 2001-2006 IBM, Inc.
# all rights reserved.
#

default_port="5555"

#
# Figure out whether we'll be using ps -ef or ps augwwwx when we want to
# search for a process.
#
if ps -ef >/dev/null 2>&1; then
	PS_FLAGS="-ef"
else
	PS_FLAGS="augwwwx"
fi

#
# set the default install directory
#
default_dest="/usr/local/bin"
for dir in /usr/sbin /sbin /bin /usr/bin /usr/local/bin
do
	if [ -d $dir ]
	then
		default_dest=$dir
	fi
done

#
# set the $ECHO flags
#
ECHO="/bin/echo"
$ECHO -n hello | grep n > /dev/null
if [ $? -eq 0 ]
then
	ECHO_N=""
	ECHO_C="\c"
else
	ECHO_N="-n"
	ECHO_C=""
fi

#
# must be root to set up agent processes
#
id | grep root > /dev/null 2>&1
if [ $? -ne 0 ]; then
	$ECHO ""
	$ECHO "***"
	$ECHO "*** You must be root to install the IBM Rational Build Forge Agent."
	$ECHO "***"
	exit 1
fi

if [ -z "$AGENT_PORT" -a -z "$AGENT_DIR" ]; then
	cat <<EOF

Welcome to the IBM Rational Build Forge Agent installation script. This script
will install the IBM Rational Build Forge Agent Service.

Press return to continue ...
EOF
	read junk
	$ECHO ""
fi


#
# Make sure xinetd, inetd or launchctl is available.
#
iss_type=
iss_pid=
iss_file=
iss_name=
if [ -f /etc/inetd.conf ]; then
	iss_pid=`ps $PS_FLAGS | grep inetd | grep -v grep | awk 'NR==1 {print $2}'`
	if [ -n "$iss_pid" ]; then
		iss_type="inetd"
		iss_name="bfagent"
		iss_file="/etc/inetd.conf"
	fi
fi
if [ -d /etc/xinetd.d ]; then
	iss_pid=`ps $PS_FLAGS | grep xinetd | grep -v grep | awk 'NR==1 {print $2}'`
	if [ -n "$iss_pid" ]; then
		iss_type="xinetd"
		iss_name="bfagent"
		iss_file="/etc/xinetd.d/$iss_name"
	fi
fi
if [ -x /bin/launchctl -a -d /System/Library/LaunchDaemons ]; then
	iss_pid=
	iss_type="launchd"
	iss_name="com.ibm.rational.bfagent"
	iss_file="/System/Library/LaunchDaemons/${iss_name}.plist"
fi
if [ -z "$iss_type" ]; then
	$ECHO ""
	$ECHO "ERROR: No internet super-server seems to be installed!"
	$ECHO ""
	$ECHO "This script understands the inetd, xinetd, and launchd internet"
	$ECHO "super-servers, but it was not able to find any of them on this"
	$ECHO "system."
	$ECHO ""
	$ECHO "Installation aborted."
	$ECHO ""
	exit 2
fi


#
# Require license approval
#
if [ ! -f LICENSE.txt ]
then
	$ECHO ""
	$ECHO "ERROR: Unable to find license text to display."
	$ECHO "You may not install without accepting a license agreement."
	$ECHO ""
	$ECHO "Installation aborted."
	$ECHO ""
	exit 3
fi

response=
while [ -z "$response" ]; do
	$ECHO ""
	$ECHO "You must accept the license agreement to install this software."
	$ECHO "Enter nothing to display the license, or 'YES' to accept it."
	$ECHO "Any other response will terminate the installation."
	$ECHO ""
	$ECHO $ECHO_N "Do you accept this license? $ECHO_C"
	read response
	if [ -z "$response" ]; then
		cat LICENSE.txt
	fi
done

if [ "x$response" != "xYES" ]; then
	$ECHO ""
	$ECHO "You must accept the license agreement to install this software."
	$ECHO ""
	$ECHO "Installation aborted."
	$ECHO ""
	exit 4
fi


#
# get the port number for the service
#
port=""
if [ -n "$AGENT_PORT" ]; then
	port=$AGENT_PORT
fi

while [ -z "$port" ]; do
	$ECHO ""
	$ECHO "Enter the port number the IBM Rational Build Forge Agent"
	$ECHO "should listen on.  The port number must match the port"
	$ECHO "number defined in the IBM Rational Build Forge Management"
	$ECHO "Console.  Press return for the default port number of"
	$ECHO "$default_port."
	$ECHO ""
	$ECHO $ECHO_N "Port number [$default_port] $ECHO_C"
	read port
	if [ -z "$port" ]; then
		port=$default_port
	fi
	grep -v bfagent /etc/services | grep "$port/tcp" > /dev/null 2>&1
	serv=`grep "[ 	]$port/tcp" /etc/services | grep -v "^#" | grep -v "^bfagent[ 	]" | awk 'NR==1 {print $1}'`
	if [ -n "$serv" ]; then
		$ECHO ""
		$ECHO "The \"$serv\" service appears to be using port \"$port\"."
		$ECHO $ECHO_N "Use this port number anyway? (y/[n]) $ECHO_C"
		read ok
		ok=`$ECHO "$ok" | cut -c1 | tr Y y`
		if [ -z "$ok" -o "x$ok" != "xy" ]; then
			port=
		fi
	fi
done

#
# get the destination dir for the service
#
dest=""
if [ -n "$AGENT_DIR" ]; then
	dest=$AGENT_DIR
fi
while [ -z "$dest" ]
do
	$ECHO ""
	$ECHO "Enter the directory where the IBM Rational Build Forge Agent"
	$ECHO "should be installed.  Press return for the default of"
	$ECHO "$default_dest"
	$ECHO ""
	$ECHO $ECHO_N "Installation directory [$default_dest] $ECHO_C"
	read dest
	if [ -z "$dest" ]; then
		dest=$default_dest
	fi
	if [ ! -d "$dest" ]; then
		$ECHO ""
		$ECHO "The directory \"$dest\" does not exist."
		$ECHO $ECHO_N "Create it? ([y]/n)  $ECHO_C"
		read ok
		ok=`$ECHO "$ok" | cut -c1 | tr N n`
		if [ -z "$ok" -o "x$ok" != "xn" ]; then
			mkdir -p "$dest" > /dev/null 2>&1
			if [ -d "$dest" ]; then
				$ECHO "The \"$dest\" directory was created."
			else
				$ECHO "Unable to create \"$dest\"."
				default_dest=$dest
				dest=""
			fi
		else
			default_dest=$dest
			dest=""
		fi
	fi
done

#
# copy the BF agent service
#
set -e
cp bfagent "$dest"
chmod 700 "$dest/bfagent"
chown root "$dest/bfagent"
set +e
chgrp bin "$dest/bfagent" > /dev/null 2>&1
$ECHO "IBM Rational Build Forge Agent installed in $dest"

#
# make a serial number for this install to save files as
#
ser=`date "+%Y%m%d%H%M"`."$$"

#
# copy the BF agent config file
#
conf_real="/etc/bfagent.conf"
conf_orig="/etc/bfagent.conf-example"
conf_new="bfagent.conf"

set -e
cp "$conf_new" "$conf_orig"
chmod 640 "$conf_orig"
chown root "$conf_orig"
set +e
chgrp bin "$conf_orig" >/dev/null 2>&1
$ECHO "Created \"$conf_orig\" configuration file example."

if [ -f "$conf_real" ]; then
	$ECHO "Configuration file \"$conf_real\" already exists."
else
	set -e
	cp "$conf_orig" "$conf_real"
	set +e
	$ECHO "Created \"$conf_real\" configuration file using the example."
fi


#
# update the update_path variable in the config file
#
if grep '^update_path' "$conf_real" >/dev/null 2>&1; then
	cp "$conf_real" "$conf_real.$ser"
	grep -v '^update_path' "$conf_real.$ser" > "$conf_real"
fi
set -e
$ECHO "update_path $dest/bfagent" >> "$conf_real"
chmod 640 "$conf_real"
chown root "$conf_real"
set +e
chgrp bin "$conf_real" >/dev/null 2>&1
rm -f "$conf_real.$ser"


#
# add buildforge to the /etc/services file if needed
#
if grep "^bfagent[ 	]" /etc/services >/dev/null 2>&1; then
	cp /etc/services /etc/services.$ser
	grep -v "^bfagent[ 	]" /etc/services.$ser > /etc/services
fi
set -e
$ECHO "bfagent	$port/tcp	# IBM Rational Build Forge Agent Service" >> /etc/services
set +e
$ECHO "IBM Rational Build Forge agent added to /etc/services on port $port"
rm -f "/etc/services.$ser"


#
# set up launchd if it's present
#
case $iss_type in
	launchd)
		#
		# There may be a xinetd.d file, since that is how we used to do
		# things.  This could cause problems.  If it exists, then blow it
		# away and HUP xinetd.
		#
		if [ -f /etc/xinetd.d/bfagent ]; then
			$ECHO "Deleted old service definition at /etc/xinetd.d/bfagent"
			rm -f /etc/xinetd.d/bfagent
			xinetd_pid=`ps $PS_FLAGS | grep xinetd | grep -v grep | awk 'NR==1 { print $2 }'`
			if [ -n "$xinetd_pid" ]; then
				kill -HUP "$xinetd_pid"
			fi
		fi

		#
		# If we already have a .plist file, then replace it.  The agent may
		# have moved...
		#
		if [ -f "$iss_file" ]; then
			/bin/launchctl stop "$iss_name"
			/bin/launchctl unload "$iss_file"
			/bin/rm -f "$iss_file"
			$ECHO "Replacing service definition for $iss_name"
		fi

		#
		# Create the new .plist and install it
		#
		set -e
		(
			cat <<EOF
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">
<plist version="1.0">
<dict>
        <key>InitGroups</key>
        <true/>
        <key>Label</key>
        <string>com.ibm.rational.bfagent</string>
        <key>ProgramArguments</key>
        <array>
                <string>_BF_BINDIR_/bfagent</string>
        </array>
        <key>Sockets</key>
        <dict>
                <key>Listeners</key>
                <dict>
                        <key>SockServiceName</key>
                        <string>bfagent</string>
                </dict>
        </dict>
        <key>inetdCompatibility</key>
        <dict>
                <key>Wait</key>
                <false/>
        </dict>
</dict>
</plist>
EOF
		) \
		| sed -e "s,_BF_BINDIR_,$dest,g" \
		> "$iss_file"
		/bin/launchctl load "$iss_file"
		/bin/launchctl start "$iss_name"
		set +e
		$ECHO "IBM Rational Build Forge Agent Service definition created as $iss_name"
		;;

	xinetd)
		set -e
		(
			cat <<EOF
# default: on
# description: The IBM Rational Build Forge Agent serves build requests
#    from the IBM Rational Build Forge Management Console.
service bfagent
{
	disable		= no
	flags		= REUSE
	socket_type	= stream        
	wait		= no
	user		= root
	server		= _BF_BINDIR_/bfagent
	flags 		= IPV6
	log_on_failure	+= USERID
}
EOF
		) \
		| sed -e "s,_BF_BINDIR_,$dest,g" \
		> /etc/xinetd.d/bfagent
		set +e
		$ECHO "IBM Rational Build Forge Agent Service definition created as $iss_file"
		;;

	inetd)
		cp /etc/inetd.conf /etc/inetd.conf.$ser
		grep -v "^bfagent[ 	]" /etc/inetd.conf.$ser > /etc/inetd.conf
		set -e
		$ECHO "bfagent stream tcp nowait root _BF_BINDIR_/bfagent bfagent" \
			| sed -e "s,_BF_BINDIR_,$dest,g" \
			>> /etc/inetd.conf
		$ECHO "bfagent stream tcp6 nowait root _BF_BINDIR_/bfagent bfagent" \
			| sed -e "s,_BF_BINDIR_,$dest,g" \
			>> /etc/inetd.conf
		set +e
		$ECHO "IBM Rational Build Forge Agent definition added to $iss_file"
		if [ -s /etc/inetd.conf ]; then
			rm -f /etc/inetd.conf.$ser
		else
			$ECHO "ERROR: Unable to update /etc/inetd.conf"
			$ECHO "Original file is at /etc/inetd.conf.$ser"
		fi
		;;
esac

# Make sure we have a PAM configuration if PAM seems to be there.  We will
# use sshd's entries as a template if they are there; otherwise, fall back
# on login's.
if [ -d /etc/pam.d ]; then
	if [ ! -f /etc/pam.d/bfagent ]; then
		set -e
		if [ -f /etc/pam.d/sshd ]; then
			cp /etc/pam.d/sshd /etc/pam.d/bfagent
		else
			cp /etc/pam.d/login /etc/pam.d/bfagent
		fi
		set +e
		$ECHO "***"
		$ECHO "*** Please verify that the PAM entries in /etc/pam.d/bfagent"
		$ECHO "*** conform to your local security policies."
		$ECHO "***"
	fi
elif [ -f /etc/pam.conf ]; then
	cp /etc/pam.conf /etc/pam.conf.$ser
	grep -v bfagent /etc/pam.conf.$ser > /etc/pam.conf

	# duplicate either the sshd or login auth entries
	if grep "^sshd[ 	][ 	]*auth[ 	]" /etc/pam.conf.$ser >/dev/null 2>&1; then
		copy_from=sshd
	else
		copy_from=login
	fi
	grep "^${copy_from}[ 	][ 	]*auth[ 	]" /etc/pam.conf.$ser \
		| sed -e "s/^${copy_from}/bfagent/" \
		| grep -v pam_dial_auth \
		>> /etc/pam.conf

	# Now do the same for the account entries ...
	if grep "^sshd[ 	][ 	]*account[ 	]" /etc/pam.conf.$ser >/dev/null 2>&1; then
		copy_from=sshd
	else
		copy_from=login
	fi
	grep "^${copy_from}[ 	][ 	]*account[ 	]" /etc/pam.conf.$ser \
		| sed -e "s/^${copy_from}/bfagent/" \
		| grep -v pam_dial_auth \
		>> /etc/pam.conf

	$ECHO "***"
	$ECHO "*** Please verify that the bfagent PAM entries in /etc/pam.conf"
	$ECHO "*** conform to your local security policies."
	$ECHO "***"
	if [ -s /etc/pam.conf ]; then
		rm -f /etc/pam.conf.$ser
	else
		$ECHO "ERROR: unable to update /etc/pam.conf"
		$ECHO "Original file is at /etc/pam.conf.$ser"
		exit 1
	fi
fi


if [ -n "$iss_pid" ]; then
	if kill -HUP "$iss_pid"; then
		$ECHO "Sent SIGHUP to $iss_type process $iss_pid"
	else
		$ECHO "***"
		$ECHO "*** Unable to send $iss_type process $iss_pid a SIGHUP."
		$ECHO "*** You must restart it for the Agent to become active."
		$ECHO "***"
	fi
fi

$ECHO "IBM Rational Build Forge Agent Installation Complete."

exit 0

